<?php

namespace Drupal\google_photos_api\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Settings form for Google Photos.
 */
class GooglePhotosSettingsForm extends ConfigFormBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a new SiteConfigureForm.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManagerInterface $entityTypeManager) {
    parent::__construct($config_factory);
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'google_photos_api_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return [
      'google_photos_api.settings',
    ];
  }

  /**
   * Build Admin Settings Form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('google_photos_api.settings');

    $form['google_photos_api_settings'] = [
      '#type' => 'details',
      '#title' => $this->t('Google Photos Api Settings'),
      '#open' => TRUE,
    ];
    $form['google_photos_api_settings']['google_api_client_id'] = [
      '#type' => 'hidden',
      '#default_value' => $config->get('google_api_client_id'),
    ];
    $google_api_client_id = $config->get('google_api_client_id');
    if ($google_api_client_id) {
      $google_api_client = $this->entityTypeManager->getStorage('google_api_client')->load($google_api_client_id);
      if ($google_api_client->getAuthenticated()) {
        $this->messenger()->addMessage($this->t('Google account is authenticate, ready to call api'));
      }
      else {
        $authenticate_link = Link::createFromRoute($this->t('Authenticate now'),
          'google_api_client.callback',
          [
            'id' => $google_api_client_id,
            'destination' => '/admin/config/services/google_photos_api',
          ])->toString();
        $this->messenger()->addWarning($this->t('You need to authenticate with google, %link', ['%link' => $authenticate_link]));
      }
    }

    $form['google_photos_api_settings']['client_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Google Photos Api ClientID'),
      '#required' => TRUE,
      '#default_value' => $google_api_client_id ? $google_api_client->getClientId() : '',
    ];
    $form['google_photos_api_settings']['client_secret'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Google Photos Api Client Secret'),
      '#required' => TRUE,
      '#default_value' => $google_api_client_id ? $google_api_client->getClientSecret() : '',
    ];
    $selected_scopes = [];
    $all_scopes = google_api_client_google_services_scopes('photoslibrary');
    if ($google_api_client_id) {
      $selected_scopes = array_intersect($all_scopes['photoslibrary'], $google_api_client->getScopes());
      $selected_scopes = array_keys($selected_scopes);
    }
    $form['google_photos_api_settings']['scopes'] = [
      '#type' => 'select',
      '#title' => $this->t('Scopes'),
      '#default_value' => $selected_scopes,
      '#options' => $all_scopes,
      '#required' => TRUE,
      '#multiple' => TRUE,
      '#description' => $this->t('Select scopes which you want to authenticate for the api.'),
    ];
    $form['google_photos_api_settings']['access_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Access Type'),
      '#default_value' => $google_api_client_id ? $google_api_client->getAccessType() : TRUE,
      '#options' => [$this->t('Online'), $this->t('Offline')],
      '#required' => TRUE,
      '#description' => $this->t('Access type defines whether the site can save access token and use later i.e. Offline.'),
    ];

    $form['google_photos_api_settings']['redirect_url'] = [
      '#type' => 'markup',
      '#markup' => $this->t('Google Photos Api Redirect URL') . '<br/><code>' . google_api_client_callback_url() . '</code><br/>' . $this->t('Redirect URL to be pasted in the google api console.'),
    ];
    return parent::buildForm($form, $form_state);
  }

  /**
   * Build Admin Submit.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    if ($values['google_api_client_id']) {
      $google_api_client = $this->entityTypeManager->getStorage('google_api_client')->load($values['google_api_client_id']);
      $google_api_client->setClientId($values['client_id']);
      $google_api_client->setClientSecret($values['client_secret']);
      $google_api_client->setAccessType($values['access_type']);
      $google_api_client->setScopes($values['scopes']);
    }
    else {
      $account = [
        'name' => 'Google Photos Api Client',
        'client_id' => $values['client_id'],
        'client_secret' => $values['client_secret'],
        'access_token' => '',
        'services' => ['photoslibrary'],
        'is_authenticated' => FALSE,
        'access_type' => $values['access_type'],
      ];
      $google_api_client = $this->entityTypeManager->getStorage('google_api_client')->create($account);
      $google_api_client->setScopes($values['scopes']);
    }
    $google_api_client->save();

    if ($google_api_client->getId()) {
      $this->config('google_photos_api.settings')
        ->set('google_api_client_id', $google_api_client->getId())
        ->save();
    }
    parent::submitForm($form, $form_state);
  }

}
